#!/usr/bin/env python -O

# Top level script for invoking Wing IDE.  If --use-src is specified
#  as an arg, then the files in WINGHOME/src, WINGHOME/external,
#  WINGHOME/opensource will be used; otherwise, the files in the version
#  specific bin directory will be used if it exists.
#
# Copyright (c) 2000-2001, Archaeopteryx Software, Inc.  All rights reserved.
#
# Written by Stephan R.A. Deibel and John P. Ehresman

import sys
import os
import string

# This lets us debug the IDE with an installed copy of itself on win32
if '--use-devel-winghome' in sys.argv:
  i = sys.argv.index('--use-devel-winghome')
  WINGHOME=sys.argv[i + 1]
  del sys.argv[i:i + 2]
  
# This is used on win32
elif '--use-winghome' in sys.argv:
  i = sys.argv.index('--use-winghome')
  WINGHOME=sys.argv[i + 1]
  del sys.argv[i:i + 2]
  
# This is for development on linux
else:
  WINGHOME='/usr/lib/wingide'
  
# Some internal code still depends on this being set in the environment
os.environ['WINGHOME'] = WINGHOME

# Caller requested forcing execution from source code even if binaries are available
if '--use-src' in sys.argv:
  base_path = WINGHOME
  sys.stderr.write("Running Wing from source code in " + base_path + "\n")
  
# Otherwise locate the wing executable base path:  We look through bin dir
# in sub-directory according to the Python version that's running, and
# try all lesser releases with same major.minor number if we don't
# have an exact match
else:
  major = (sys.hexversion & 0xff000000) >> 24
  minor = (sys.hexversion & 0x00ff0000) >> 16
  release = (sys.hexversion & 0x0000ff00) >> 8
  sys.stderr.write("Running Wing IDE with Python version %d.%d.%d\n" % (major, minor, release))
  success = 0
  while not success and release >= 0:
    version_id = '%d.%d.%d' % (major, minor, release)
    base_path = os.path.join(WINGHOME, 'bin',  version_id)
    if os.path.isdir(base_path):
      success = 1
      sys.stderr.write("Using precompiled binaries for Python version " + version_id + "\n")
    else:
      release = release - 1
      
  # If not found, then try to run from source code
  if not success:
    base_path = WINGHOME
    sys.stderr.write("Cannot find Wing binaries for this version of Python!\n")
    sys.stderr.write("Trying to run from source code instead.\n")

# Check whether we managed to find a valid base path
if not os.path.isdir(base_path):
  sys.stderr.write("Running with invalid WINGHOME: " + base_path + "\n")
  sys.stderr.write("Please check your installation\n")
  sys.exit(-1)

# Remove used up args from list
if '--use-src' in sys.argv:
  sys.argv.remove('--use-src')

# Set up the import path relative to given base path
sys.path.insert(0, os.path.join(base_path, 'src'))
sys.path.insert(0, os.path.join(base_path, 'opensource'))
sys.path.insert(0, os.path.join(base_path, 'external'))
sys.path.insert(0, os.path.join(base_path, 'external', 'pygtk-0.6.5-wing'))

# If gtk location has been set up (Linux only) then notify user
gtk_prefix = os.environ.get('WING_GTK_PREFIX')
if gtk_prefix != None:
  gtk_version = string.strip(os.popen('gtk-config --version').read())
  print "Using private copy of GTK %s at %s" % (gtk_version, gtk_prefix)
  gtk_exe_prefix = os.environ.get('GTK_EXE_PREFIX')
  print "GTK themes have been disabled"
#  if gtk_exe_prefix != None:
#    print "GTK themes will be read from the installation at %s" % gtk_exe_prefix
elif sys.platform != 'win32':
  gtk_prefix = string.strip(os.popen('gtk-config --prefix').read())
  gtk_version = string.strip(os.popen('gtk-config --version').read())
  print "Using system-wide GTK %s at %s" % (gtk_version, gtk_prefix)
  
# Check if a display has been specified
if sys.platform != 'win32' and not os.environ.has_key("DISPLAY") \
   and '--display' not in sys.argv:
  print "No X Windows DISPLAY has been specified"
  print "Exiting Wing IDE"

# Run the IDE / check for failure to open display
else:
  try:
    import main
    orig_execpath = os.environ.get('ORIG_PATH')
    orig_libpath = os.environ.get('ORIG_LD_LIBRARY_PATH')
    main.main(orig_execpath, orig_libpath)
  except '':
    print "Fatal startup error:  May not be able to open X Windows display"
    print "Exiting Wing IDE"

