#!/usr/bin/env python
#########################################################################
""" wingdb.py    -- Top-level command line for networked debug server 
                    that supports remote invocation and control 
                    of a debug process

Copyright (c) 2000-2001, Archaeopteryx Software, Inc.  All rights reserved.

Written by Stephan R.A. Deibel and John P. Ehresman

"""
#########################################################################

print "ENTERING WINGDB.PY"

# Save previously loaded modules
import sys
import copy_reg # cPickle is screwed up if we don't hold onto this
prev_mods = sys.modules.keys()

# Set wait-on-exit default so we can do something reasonable if the
# debug process totally fails to start up
if '--wait-on-exit' in sys.argv:
  waitonexit = 1
else:
  waitonexit = 0
version_id = "UNKNOWN"
kSrcPath = "UNKNOWN"
kBinPath = "UNKNOWN"
WINGHOME = "UNKNOWN"

# Import what we need locally
import os
import string
import traceback

try:
  
  # Figure out the location that we should load modules from
  if '--use-winghome' in sys.argv:
    i = sys.argv.index('--use-winghome')
    WINGHOME=sys.argv[i + 1]
    del sys.argv[i:i + 2]
  else:
    WINGHOME='/usr/lib/wingide'
    
  # Work around win32 path joining problems
  if sys.platform == 'win32' and WINGHOME[-1] == '\\':
    WINGHOME = WINGHOME[:-1]

  # Some internal code still depends on this being set in the environment
  os.environ['WINGHOME'] = WINGHOME

  # Try to locate the debugger modules in binary form first:  We look
  # in directory according to the Python version that's running, and
  # try all lesser releases with same major.minor number if we don't
  # have an exact match
  major = (sys.hexversion & 0xff000000) >> 24
  minor = (sys.hexversion & 0x00ff0000) >> 16
  release = (sys.hexversion & 0x0000ff00) >> 8
  success = 0
  while not success and release >= 0:
    version_id = '%d.%d.%d' % (major, minor, release)
    sys.path[0] = os.path.join(WINGHOME, 'bin',  version_id, 'src', 'debug', 'server')
    try:
      import netserver
      import abstract
      success = 1
    except ImportError:
      release = release - 1

  # Try to locate the debugger modules in source form if the binary
  # form could not be found
  if not success:
    sys.path[0] = os.path.join(WINGHOME, 'src', 'debug', 'server')
    import netserver
    import abstract    
  
  # Unload modules so we can detect debuggee imports
  for key in sys.modules.keys():
    if not key in prev_mods:
      del sys.modules[key]

# Always wait for user to see errors in above code
except:
  import traceback
  traceback.print_exc(sys.exc_info())
  print "Error: Failed to start the debug server"
  if os.environ.has_key('DEVEL_WINGHOME'):
    print "Error: You may need to undefined or alter the DEVEL_WINGHOME env"
  print "Error: You may be running an unsupported version of Python"
  print "Python version =", version_id
  print "WINGHOME=", WINGHOME
  if waitonexit:
    line = raw_input("-- Type return or enter to exit --")
  sys.exit(-1)
  
#########################################################################
# Utilities
#########################################################################

def __PrintUsage(err):
  err.out("usage: wingdb.py host:port <attachport> <stop arg> <silent arg> <wait arg> <program> [arg]*")


#########################################################################
# Execution starts here
#########################################################################

if __name__=='__main__':

  # Make sure we always give opportunity to see output when wait-on-exit is true!
  try:
    
    # Use stderr for outputting errors
    err = abstract.CErrStream(sys.stderr)
  
    # Message of how to get help
    get_help_msg = "Type 'wingdb.py help' for usage info"
  
    # Check args
    if len(sys.argv) > 1 and sys.argv[1] == 'help':
      __PrintUsage(err)
      err.out("""
This is the same as executing 'program.py [arg]*' on the command line
except that the program is run in the Archaeopteryx Wing debugger.

Host:port indicate location where the previously started debug client 
(Wing IDE) is listening for reverse connection from this server.

<attachport> is the port on which the debug server will listen for client
requests to attach to an already-running process when it fails to
attach initially to the given debug host/port.  Note that this requires
that the .wingdebugpw file is present, or all attach requests will
be refused.

<stop arg> is one of --first-stop to stop on the first line of the
debug program, or --no-first-stop to run to first breakpoint or
completion.

<silent arg> is one of --silent to run silently without any extra
output (other than that which your program produces), or --verbose
to include debugger status messages.

<wait arg> is one of --wait-on-exit or --nowait-on-exit.  When set to
wait-on-exit, the debugger will wait for user to hit a key before exiting
entirely (this is useful so output doesn't disappear immediately).

If program.py is omitted, then the server connects back to the client but
waits for the client to issue commands before a program is started.  This
mode allows reuse of the server for running multiple programs.

Otherwise, program.py is started immediately with given arguments and the
debug session will be ended as soon as the script has exited.  This mode
is appropriate for debugging externally triggered programs, such as CGI
scripts for web services.  Note that failure to connect to the IDE in this
mode will result in running the program stand-alone without debugging it.
  """)
      sys.exit(0)
    if not sys.argv[5:]:
      __PrintUsage(err)
      err.out("Required args    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^")
      err.out(get_help_msg)
      sys.exit(2)
  
    # Get host/port info for connection
    try:
      hostport = sys.argv[1]
      colonpos = string.index(hostport, ':')
      host = hostport[0:colonpos]
      if host == '':
        host = '127.0.0.1'
      port = int(hostport[colonpos+1:len(hostport)])
    except:
      __PrintUsage(err)
      err.out("Could not parse  ^^^^^^^^^")
      err.out(get_help_msg)
      sys.exit(2)
  
    # Get attach port used for attach requests by debug client
    try:
      attachport = int(sys.argv[2])
    except:
      __PrintUsage(err)
      err.out("Could not parse            ^^^^^^^^^^^^")
      err.out(get_help_msg)
      sys.exit(2)
    
    # Get whether or not to stop on first line
    try:
      firststop = sys.argv[3]
      if firststop == "--first-stop":
        firststop = 1
      elif firststop == "--no-first-stop":
        firststop = 0
      else:
        raise ValueError
    except (ValueError, KeyError):
      __PrintUsage(err)
      err.out("Could not parse                           ^^^^^^^^^^")
      err.out(get_help_msg)
      sys.exit(2)
  
    # Get whether to run silently
    try:
      silent = sys.argv[4]
      if silent == "--silent":
        silent = 1
      elif silent == "--verbose":
        silent = 0
      else:
        raise ValueError
    except (ValueError, KeyError):
      __PrintUsage(err)
      err.out("Could not parse                                      ^^^^^^^^^^^^")
      err.out(get_help_msg)
      sys.exit(2)
  
    # Get whether to wait on exit
    try:
      waitonexit = sys.argv[5]
      if waitonexit == "--wait-on-exit":
        waitonexit = 1
      elif waitonexit == "--nowait-on-exit":
        waitonexit = 0
      else:
        raise ValueError
    except (ValueError, KeyError):
      __PrintUsage(err)
      err.out("Could not parse                                                   ^^^^^^^^^^")
      err.out(get_help_msg)
      sys.exit(2)
  
    # Get program name
    try:
      extrafnargs = 0
      filename = sys.argv[6]
      if len(filename) > 1 and filename[0] == '"':
        while len(filename) <= 2 or filename[-2] == '\\' or filename[-1] != '"':
          if not sys.argv[6+extrafnargs+1]:
            err.out('Error:', 'Filename not found or malformed')
            sys.exit(1)
          filename = filename + " " + sys.argv[6+extrafnargs+1]
          extrafnargs = extrafnargs + 1
        filename = filename[1:-1]
        filename = string.replace(filename, '\"', '"')
      if not os.path.exists(filename):
        err.out('Error:', filename, 'does not exist')
        sys.exit(1)
  
      # Prune args down to just args for the debugged program
      # By convention we leave in the program name since an argv
      # length zero is problematic for some code (like gtk)
      del sys.argv[0]
      del sys.argv[0]
      del sys.argv[0]
      del sys.argv[0]
      del sys.argv[0]
      del sys.argv[0]
      del sys.argv[0]
      for i in range(0, extrafnargs):
        del sys.argv[0]
  
    # No program given
    except:
      err.out('Error:', 'Filename not found or malformed')
      sys.exit(1)
  
    # Turn off any further output if running silently
    if silent:
      err = abstract.CNullErrStream()
  
    # Run the session
    try:
  
      # Run the debug session in server-side initiated mode
      if filename != '':
        filename = os.path.abspath(filename)
        err.out("Starting debug session for", filename)
        err.out("Network peer is ", host, "port", port)
        err.out("Attach port = ", attachport)
        err.out("Initial stop = ", firststop)
        server = netserver.CNetworkServer(host, port, attachport, err, firststop)
        os.environ['WINGDB_ACTIVE'] = "1"
        server.Run(filename, sys.argv)
  
      # Run debug session in client-side initiated mode
      # NOTE: This is broken due to internal changes; not critical because
      # it is never used at this point and should be redesigned so that
      # debugging is started by another listener process rather than
      # trying to support listen here : See the comment in netserver's 
      # Listen() method! [SRAD]
      else:
        err.out("Starting debug session and listening for client commands")
        err.out("Network peer is ", host, "port", port)
        err.out("Attach port = ", attachport)
        err.out("Initial stop = ", firststop)
        os.environ['WINGDB_ACTIVE'] = "1"
        server = netserver.CNetworkServer(host, port, attachport, err, firststop)
        server.Listen()
  
    # Cook exceptions for better display
    except:
      nt, val, tb = sys.exc_info()
      err.out("Server exiting abnormally on exception")
      err.out("Exception:", nt)
      err.out("Values:", val)
      tbf = traceback.format_tb(tb)
      for item in tbf:
        err.out(item)
  
    # Explicitely stop the server to make sure socket is closed
    try:
      server.Stop()
    except:
      pass
    server = None

  # Always wait for user to exit if requested
  finally:
    if waitonexit:
      line = raw_input("-- Type return or enter to exit --")
    
