#########################################################################
""" _extensions.py -- Lists the modules to import for supported
                      mainloop environments

Copyright (c) 1999-2001, Archaeopteryx Software, Inc.  All rights reserved.

Written by Stephan R.A. Deibel and John P. Ehresman

This module just enumerates the supported environments where the debug
server can register its communication sockets in order to more reliably
service those sockets.  See netserver.py for details on the sockets
used and the protocol between the debug server and client.

To add your own environment to the supported list, you need to:
  

(1) Write a module that implements the necessary hooks for registering
    a socket in the environment.  The module must contain the following 
    at the top level scope:
  
    (a) kIndicatorModuleName : A string containing the name of the module
    whose import indicates that your supported mainloop environment is
    present in the debug program.  E.g., for gtk this is 'gtk', and for
    Zope this is 'ZServer'.

    (b) _SocketHook : A class that implements support for registering
    and deregistering sockets with your mainloop environment.  This is
    a descendent of the class _SocketHook defined below.

(2) Place this module into this directory (src/debug/server)

(3) Add your module's name to the list below
    
"""
#########################################################################

# To add a supported mainloop environment, add your module name to
# the list below.
kSupportedMainloops = [ '_gtkhooks', '_zopehooks', '_tkinterhooks', '_qthooks', 
                        '_wxhooks' ]


#########################################################################
# Abstract class for managing the debug server sockets
#########################################################################
class _SocketHook:
  """ Class for managing the debug server sockets:  Descendents implement
  registering sockets with resident mainloop environments in order to
  cause them to be serviced more reliably.  This class is defined only
  to document what is required of mainloop extension modules. """

  #----------------------------------------------------------------------
  def __init__(self, err):
    """ Constructor : err is the error output stream, which has a single
    method 'out' that acts like Python's 'print' command (but with normal
    function syntax, accepting 1 or more comma seperated args).  This
    can be used to print messages during debugging your environment
    support and will remain silent unless the verbose debugging option
    is turned on. """
    
    self.fErr = err
    
  #-----------------------------------------------------------------------
  def _Setup(self, mod, s, cb_fct):
    """ Attempt to set up socket registration with the given module
    reference : This should be a reference to the indicator module
    for the supported environment.  The first socket is registered 
    with given action callback via _RegisterSocket().  Returns the
    socket if succeeded or None if fails (e.g. because the module is 
    not yet fully loaded and we cannot yet use it to start registering 
    sockets).  Note that the returned socket may be different than the
    socket passed in because some environments require a wrapper:  The
    returned socket is then used in place of the original in the
    debug server code. """
    pass
    
  #----------------------------------------------------------------------
  def _RegisterSocket(self, s, cb_fct):
    """ Function to register a socket with a mainloop: Subsequently the given
    callback function is called whenever there is data to be read on the
    socket.  Returns the socket if succeeded; None if fails. As in _Setup(),
    the returned socket may differ from the one passed in, in which case
    the debug server will substitute the socket that is used in its code."""
    pass
    
  #----------------------------------------------------------------------
  def _UnregisterSocket(self, s):
    """ Function to unregister a socket with the supported environment.
    The socket passed in should be the one returned from _Setup() or
    _RegisterSocket(). """    
    pass

    