#########################################################################
""" _wxhooks.py -- wxPython socket management hooks for the Wing IDE debugger

Copyright (c) 1999-2001, Archaeopteryx Software, Inc.  All rights reserved.

Written by Stephan R.A. Deibel and John P. Ehresman

"""
#########################################################################

import _extensions
import types

# The name of the module to watch for that indicates presence of this
# supported mainloop environment
kIndicatorModuleName = 'wxPython.wx'

# The hook is activated periodically -- timeout (in milliseconds) to poll
kPollTimeout = 500

#########################################################################
# wxPython-specific support for managing the debug server sockets
#########################################################################
class _SocketHook(_extensions._SocketHook):
  """ Class for managing the debug server sockets:  This is used only
  when wxPython is detected as being present in the debuggee's code. """

  #----------------------------------------------------------------------
  def __init__(self, err):
    """ Constructor """
    _extensions._SocketHook.__init__(self, err)
    self.__fWxModule = None
    self.__fTimer = None
    
  #-----------------------------------------------------------------------
  def _Setup(self, mod, s, cb_fct):
    """ Attempt to set up socket registration with the given module
    reference : This should be a reference to the indicator module
    for the supported environment.  The first socket is registered 
    with given action callback via _RegisterSocket().  Returns the
    socket if succeeded or None if fails (e.g. because the module is 
    not yet fully loaded and we cannot yet use it to start registering 
    sockets.  Note that the returned socket may be different than the
    socket passed in because some environments require a wrapper:  The
    returned socket is then used in place of the original in the
    debug server code. """
    
    # Check if we can pull this off now (if not, we keep trying)
    if mod.__name__ != kIndicatorModuleName:
      return None

    # Try to register the first socket
    self.__fWxModule = mod
    new_sock = self._RegisterSocket(s, cb_fct)
    if new_sock == None:
      self.__fWxModule = None
      return None
    
    # Success
    return new_sock
  
  #----------------------------------------------------------------------
  def _RegisterSocket(self, s, cb_fct):
    """ Function to register a socket with a mainloop: Subsequently the given
    callback function is called whenever there is data to be read on the
    socket.  Returns the socket if succeeded; None if fails. As in _Setup(),
    the returned socket may differ from the one passed in, in which case
    the debug server will substitute the socket that is used in its code."""

    # Try to use given module to setup timeout
    try:
      if self.__fWxModule == None or not hasattr(self.__fWxModule, 'wxTimer'):
        return None
         
      timer_class = getattr(self.__fWxModule, 'wxTimer')
      if not isinstance(timer_class, types.ClassType):
        return None

      # Create a wxPython timer -- derive a class at runtime because we
      #  don't have access to the base class until now
      class CTimerAdapter(timer_class):
        def Notify(self):
          self._fCallback()
          
      self.__fTimer = CTimerAdapter()
      self.__fTimer._fCallback = self.__TimerCallback
      self.__fTimer.Start(kPollTimeout, 0)
      
      self.fErr.out("################## Timer for polling socket registered with wxPython: ", s)
      self.__fCallBack = cb_fct
      
      return s

    # Failed but keep checking
    except:
      import traceback
      traceback.print_exc()
      self.fErr.out("################## 'wxPython' module not fully loaded")
      return None
    
  #----------------------------------------------------------------------
  def _UnregisterSocket(self, s):
    """ Function to unregister a socket with the supported environment.
    The socket passed in should be the one returned from _Setup() or
    _RegisterSocket(). """    
    
    self.__fTimer.Stop() 
    self.__fTimer = None
    self.fErr.out("################ Deregistered socket with wxPython: ", s)

  #----------------------------------------------------------------------
  def __TimerCallback(self, id = None):
    """ Method used as timer callback when polling is used. """
    
    if callable(self.__fCallBack):
      try:
        self.__fCallBack()
      except:
        self.fErr.out("################## Exception occurred in socket checking function")
