#########################################################################
""" wingdbstub.py    -- Debug stub for debuggifying Python programs

Copyright (c) 1999-2001, Archaeopteryx Software, Inc.  All rights reserved.

Written by Stephan R.A. Deibel and John P. Ehresman

Usage:
-----

This is the file that Wing DB users copy into their python project 
directory if they want to be able to debug programs that are launched
outside of the IDE (e.g., CGI scripts, in response to a browser page
load).

To use this, edit the configuration values below to match your 
Wing IDE installation and requirements of your project.

Then, add the following line to your code:

  import wingdbstub

Debugging will start immediately after this import statements.

Next make sure that your IDE is running and that it's configured
to do passive listening and accept passive connections from the
host the debug program will be running on.

Now, invoking your python file should run the code within the debugger.
Note, however, that Wing will not stop in the code unless a breakpoint
set set.

If the debug process is started before the IDE, or is not listening
at the time this module is imported then the program will run with
debugging until an attach request is seen.  Attaching only works 
if the .wingdebugpw file is present; see the manual for details.

One win32, you either need to edit WINGHOME in this script or
pass in an environment variable called WINGHOME that points to
the Wing IDE installation directory.

"""
#########################################################################

import sys
import os
import string


#------------------------------------------------------------------------
# Default configuration values:  Note that the named environment 
# variables, if set, will override these settings.

# Set this to 1 to disable all debugging; 0 to enable debugging
# (WINGDB_DISABLED environment variable)
kWingDebugDisabled = 0

# Host:port of the IDE within which to debug: As configured in the IDE
# (WINGDB_HOSTPORT environment variable)
kWingHostPort = 'localhost:50005'

# Port on which to listen for connection requests, so that the
# IDE can (re)attach to the debug process after it has started
# Note that a random port is used instead if this port is already
# in use!
# (WINGDB_ATTACHPORT environment variable)
kAttachPort = '50015'

# Set this to 1 to run silently without any output other than
# errors on failure to start debugging or output from your program
# (WINGDB_SILENT environment variable)
kSilent = 1

# Set this to 1 when debugging embedded scripts:  It turns off
# automatic detecting of program quit so that the debug session
# can span multiple script invocations (WINGDB_EMBEDDED environment
# variable)
kEmbedded = 0


#------------------------------------------------------------------------
# Find Wing debugger installation location

# Uncomment and set this if you are running Wing from source code
#os.environ['DEVEL_WINGHOME'] = WINGHOME

# Winghome passed by environment variable
if os.environ.has_key('WINGHOME'):
  WINGHOME=os.environ['WINGHOME']
else:
  WINGHOME='/usr/lib/wingide'
  os.environ['WINGHOME'] = WINGHOME

#------------------------------------------------------------------------
# Start debugging if not disabled and this module has never been imported
# before
if not os.environ.has_key('WINGDB_ACTIVE'):
  debugger = None
if not kWingDebugDisabled and not os.environ.has_key('WINGDB_DISABLED') and \
   not os.environ.has_key('WINGDB_ACTIVE'):

  # Obtain configuration for silent vs. verbose mode
  if os.environ.has_key('WINGDB_SILENT'):
    silent = os.environ['WINGDB_SILENT']
    try:
      silent = int(silent)
    except:
      silent = 1
  else:
    silent = kSilent

  # Determine remote host/port where the IDE is running
  if os.environ.has_key('WINGDB_HOSTPORT'):
    hostport = os.environ['WINGDB_HOSTPORT']
  else:
    hostport = kWingHostPort
  colonpos = string.find(hostport, ':')
  host = hostport[:colonpos]
  port = int(hostport[colonpos+1:])

  # Determine port to listen on locally for attach requests
  if os.environ.has_key('WINGDB_ATTACHPORT'):
    attachport = os.environ['WINGDB_ATTACHPORT']
  else:
    attachport = kAttachPort

  # Check if running embedded script
  if os.environ.has_key('WINGDB_EMBEDDED'):
    embedded = os.environ['WINGDB_EMBEDDED']
  else:
    embedded = kEmbedded

  # Try to locate the debugger modules in binary form first:  We look
  # in directory according to the Python version that's running, and
  # try all lesser releases with same major.minor number if we don't
  # have an exact match
  major = (sys.hexversion & 0xff000000) >> 24
  minor = (sys.hexversion & 0x00ff0000) >> 16
  release = (sys.hexversion & 0x0000ff00) >> 8
  success = 0
  while not success and release >= 0:
    version_id = '%d.%d.%d' % (major, minor, release)
    sys.path.append(os.path.join(WINGHOME, 'bin',  version_id, 'src', 'debug', 'server'))
    try:
      import netserver
      import abstract
      success = 1
    except ImportError:
      sys.path = sys.path[:-1]
      release = release - 1

  # Try to locate the debugger modules in source form if the binary
  # form could not be found
  if not success:
    sys.path.append(os.path.join(WINGHOME, 'src', 'debug', 'server'))
    import netserver
    import abstract    

  # Set up the error stream
  if silent:
    err = abstract.CNullErrStream()
  else:
    err = abstract.CErrStream(sys.stderr)

  # Start debugging
  debugger = netserver.CNetworkServer(host, port, attachport, err)
  debugger.StartDebug(stophere=0, autoquit=not embedded)
  os.environ['WINGDB_ACTIVE'] = "1"
  
